'use strict';

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var command = require('@pagedip/lib-command');
var command__default = _interopDefault(command);
var inquirer = _interopDefault(require('inquirer'));
var libLicenseTools = require('@pagedip/lib-license-tools');
var updateNotifier = _interopDefault(require('update-notifier'));
var fs = _interopDefault(require('mz/fs'));

var name = "pagedip";

var version = "5.2.1-rc.5";

var main$1 = "Usage: pagedip <command> [--debug] [--silent] [--dev] [--help]\n                         [--version]\n\n  The Pagedip CLI tool.\n\nCommands\n  install                 Install a Pagedip package by name.\n  run                     Run a Pagedip app.\n  start, server, ee       Start a basic Pagedip Enterprise server.\n  register                Register this host with a Pagedip License key.\n  license                 Verify and print information about a Pagedip license\n                          key.\n\n  This is the main Pagedip CLI tool. With this tool you can install Pagedip\n  packages, run servers, and more.\n\n  The following app names can be used as commands directly: api, auth,\n  couchdb-setup, editor, enterprise-edition, iis-install, live, setup, theme,\n  theme-server\n\n  Run --help on any command for further help documentation.\n\nOptions\n  --debug                 Show debug messages.\n  -s, --silent            All logs will be hidden.\n  --dev                   Run in development mode.\n  -h, -H, --help          Shows this message.\n  -v, -V, --version       Prints the version and exits.";

var register = "Usage: pagedip register [license] [--help]\n\n  Register this host with a Pagedip License key.\n\n  Register a machine with a valid license to interact with various restricted\n  Pagedip services.\n\nOptions\n  -h, -H, --help          Shows this message.";

var install = "Usage: pagedip install [<pkgname>...] [--help]\n\n  Install Pagedip packages.\n\n  This will install Pagedip packages and apps from the Pagedip registry. This\n  host should already be registered with a valid license key before installing\n  packages.\n\n  One or more packages can be installed at once. Postfix the package name with\n  @version to install a specific version.\n\nOptions\n  -h, -H, --help          Shows this message.";

var license = "Usage: pagedip license [license] [--help]\n\n  Verify and print information about a Pagedip license key.\n\n  This will verify a license key and print some basic information about it. If\n  you do not pass a license key, this will display info on the installed key.\n\nOptions\n  -h, -H, --help          Shows this message.";

var apps = [".babelrc", ".template", "api", "auth", "couchdb-setup", "editor", "enterprise-edition", "iis-install", "live", "setup", "theme", "theme-server"];

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

// check for updates daily
updateNotifier({
  pkg: { name: name, version: version }
}).notify();

var internalCommands = {
  "install": ["i"],
  "run": ["app"],
  "enterprise-edition": ["start", "server", "ee"],
  "setup": [],
  "register": [],
  "license": []
};

var resolveCommandName = function resolveCommandName(n) {
  if (typeof n !== "string" || !n) return;

  n = n.toLowerCase();

  for (var k in internalCommands) {
    if (k.toLowerCase() === n) return n;

    var a = internalCommands[k];
    if (Array.isArray(a) && a.includes(n)) return k;
  }

  return n;
};

var cli = command__default("pagedip", {
  module: module
}, function () {
  var _ref = _asyncToGenerator(function* (args, config, logger) {
    var ocmd = args.shift();
    var cmdname = resolveCommandName(ocmd);
    var showVersion = this.version();

    if (ocmd !== cmdname) {
      logger.debug("resolved command %s -> %s", logger.color.blue(ocmd), logger.color.blue(cmdname));
    }

    if (cmdname && !showVersion) {
      var cmd = command.get("pagedip-" + cmdname);
      if (cmd) {
        yield this.run(cmd, args, logger);
        return;
      }
    }

    if (apps.includes(cmdname)) {
      yield this.run("pagedip-run", [cmdname].concat(args), logger);
      return;
    }

    if (showVersion) {
      logger.info(name + " " + version);
    } else {
      logger.info(main$1);
    }
  });

  return function (_x, _x2, _x3) {
    return _ref.apply(this, arguments);
  };
}());

command__default("pagedip-install", {
  help: install,
  alias: {
    tag: "t"
  }
}, function () {
  var _ref2 = _asyncToGenerator(function* (args, config, logger) {
    if (!args.length) {
      logger.info(install);
      return;
    }

    var opts = config.pick("tag");

    yield require("./").install(args, _extends({}, opts, { logger: logger }));
  });

  return function (_x4, _x5, _x6) {
    return _ref2.apply(this, arguments);
  };
}());

command__default("pagedip-run", {}, function () {
  var _ref3 = _asyncToGenerator(function* (args, config, logger) {
    if (!args.length) {
      logger.info(undefined);
      return;
    }

    var _require = require("./"),
        load = _require.load;

    var cmd = void 0;

    try {
      cmd = yield load(args[0], { logger: logger });
    } catch (e) {
      if (e.code !== "MODULE_NOT_FOUND") throw e;

      var _ref4 = yield inquirer.prompt([{
        type: "confirm",
        name: "confirm",
        message: "Could not locate a package named '" + args[0] + "'. Do you want to attempt to install it?",
        default: true
      }]),
          confirm = _ref4.confirm;

      if (!confirm) throw e;

      yield this.run("pagedip-install", [args[0]], logger);
      cmd = yield load(args[0], { logger: logger });
    }

    yield this.run(cmd, args.slice(1), logger);
  });

  return function (_x7, _x8, _x9) {
    return _ref3.apply(this, arguments);
  };
}());

function printLicense(data, logger) {
  logger.log("%s\nID: %s\nRegistered To: %s\nIssued: %s\nExpires: %s", logger.color.white.bgGreen("Valid Pagedip license key."), data.id, data.user, new Date(data.issued).toString(), new Date(data.ttl).toString());
  logger.log("\n%s\n", data.payload);
}

command__default("pagedip-license", {
  help: license
}, function () {
  var _ref5 = _asyncToGenerator(function* (args, config, logger) {
    var data = yield libLicenseTools.verify(libLicenseTools.getLicense(args[0]));
    printLicense(data, logger);
  });

  return function (_x10, _x11, _x12) {
    return _ref5.apply(this, arguments);
  };
}());

command__default("pagedip-register", {
  help: register,
  alias: {
    "file": "f"
  }
}, function () {
  var _ref6 = _asyncToGenerator(function* (args, config, logger) {
    var key = void 0;
    var keyFile = config.get("file");

    if (args.length) {
      key = args[0];
    } else if (keyFile) {
      key = yield fs.readFile(keyFile, "utf-8");
    } else {
      var result = yield inquirer.prompt([{
        type: "input",
        name: "key",
        message: "License Key (paste the raw license contents here)",
        validate: function validate(v) {
          return Boolean(v.length);
        }
      }]);

      key = result.key;
    }

    if (!key) {
      logger.info(register);
      return;
    }

    var data = yield libLicenseTools.verifyInstall(key);
    logger.log("");
    printLicense(data, logger);
    logger.log(logger.color.green("Thank you for registering! You can now download and use Pagedip packages. \uD83C\uDF89\n"));
  });

  return function (_x13, _x14, _x15) {
    return _ref6.apply(this, arguments);
  };
}());

module.exports = cli;
